@@ -2,7 +2,25 @@ if(getRversion() >= '2.15.1') globalVariables("gIndex")
 
 mcoptions <- list(preschedule=FALSE, set.seed=TRUE)
 
+filterVectorLogical <- function(columnFilter,myNames){
+  if(!is.null(columnFilter)){
+    ignoreMatrix = sapply(columnFilter, function(el,namedCols){
+      grepl(el,namedCols)
+    },namedCols=myNames)
+    if(length(myNames)==1){
+      ignoreCols = any(ignoreMatrix)
+    }else{
+      ignoreCols = apply(ignoreMatrix,1,any)
+    }
+    return(ignoreCols)
+  }else{
+    return(rep(FALSE,length(myNames)))    
+  }
+}
+
+
 expString <- function(x){
+  if(is.na(x)) return("NA")
   doubleBase = .Machine$double.base
   toBase10log = x / log(10)
   toBaselog = x / log(doubleBase)
@@ -53,6 +71,7 @@ whichOmitted <- function(numerator, full){
 
 
 propErrorEst = function(logX){
+  logX = logX[!is.na(logX)]
   n = length(logX)
   logSumX = logMeanExpLogs(logX) + log(n)
   logSumX2 = logMeanExpLogs(2*logX) + log(n)
