@@ -39,6 +39,7 @@
 ##' @param whichModels which set of models to compare; see Details
 ##' @param progress if \code{TRUE}, show progress with a text progress bar
 ##' @param rscaleCont prior scale on all standardized slopes
+##' @param noSample if \code{TRUE}, do not sample, instead returning NA.
 ##' @return An object of class \code{BFBayesFactor}, containing the computed 
 ##'   model comparisons
 ##' @author Richard D. Morey (\email{richarddmorey@@gmail.com})
@@ -77,7 +78,7 @@
 ##'   \code{\link{anovaBF}} for the function similar to \code{regressionBF} for 
 ##'   ANOVA models.
 
-regressionBF <- function(formula, data, whichModels = "all", progress=TRUE, rscaleCont = "medium")
+regressionBF <- function(formula, data, whichModels = "all", progress=options()$BFprogress, rscaleCont = "medium",noSample=FALSE)
 {
   checkFormula(formula, data, analysis = "regression")
   dataTypes <- createDataTypes(formula, whichRandom=c(), data, analysis = "regression")
@@ -92,7 +93,7 @@ regressionBF <- function(formula, data, whichModels = "all", progress=TRUE, rsca
   
   if(progress) myapply = pblapply else myapply = lapply
   bfs <- myapply(models, lmBF, data = data, dataTypes = dataTypes,
-                rscaleCont = rscaleCont)
+                rscaleCont = rscaleCont,noSample=noSample)
 
   bfObj = do.call("c", bfs)
   if(whichModels=="top") bfObj = BFBayesFactorTop(bfObj)
